"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EnvelopeRecipientCreateManyRoleRequest(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class EnvelopeRecipientCreateManyAccessAuthRequest(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class EnvelopeRecipientCreateManyActionAuthRequest(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class EnvelopeRecipientCreateManyDataRequestTypedDict(TypedDict):
    email: str
    name: str
    role: EnvelopeRecipientCreateManyRoleRequest
    signing_order: NotRequired[float]
    access_auth: NotRequired[List[EnvelopeRecipientCreateManyAccessAuthRequest]]
    action_auth: NotRequired[List[EnvelopeRecipientCreateManyActionAuthRequest]]


class EnvelopeRecipientCreateManyDataRequest(BaseModel):
    email: str

    name: str

    role: EnvelopeRecipientCreateManyRoleRequest

    signing_order: Annotated[Optional[float], pydantic.Field(alias="signingOrder")] = (
        None
    )

    access_auth: Annotated[
        Optional[List[EnvelopeRecipientCreateManyAccessAuthRequest]],
        pydantic.Field(alias="accessAuth"),
    ] = None

    action_auth: Annotated[
        Optional[List[EnvelopeRecipientCreateManyActionAuthRequest]],
        pydantic.Field(alias="actionAuth"),
    ] = None


class EnvelopeRecipientCreateManyRequestTypedDict(TypedDict):
    envelope_id: str
    data: List[EnvelopeRecipientCreateManyDataRequestTypedDict]


class EnvelopeRecipientCreateManyRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    data: List[EnvelopeRecipientCreateManyDataRequest]


class EnvelopeRecipientCreateManyInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeRecipientCreateManyInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeRecipientCreateManyInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeRecipientCreateManyInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeRecipientCreateManyInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeRecipientCreateManyInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeRecipientCreateManyInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeRecipientCreateManyForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeRecipientCreateManyForbiddenIssue(BaseModel):
    message: str


class EnvelopeRecipientCreateManyForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeRecipientCreateManyForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeRecipientCreateManyForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeRecipientCreateManyForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeRecipientCreateManyForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeRecipientCreateManyUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeRecipientCreateManyUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeRecipientCreateManyUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeRecipientCreateManyUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeRecipientCreateManyUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeRecipientCreateManyUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeRecipientCreateManyUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeRecipientCreateManyBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeRecipientCreateManyBadRequestIssue(BaseModel):
    message: str


class EnvelopeRecipientCreateManyBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeRecipientCreateManyBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeRecipientCreateManyBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeRecipientCreateManyBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeRecipientCreateManyBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeRecipientCreateManyRoleResponse(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class EnvelopeRecipientCreateManyReadStatus(str, Enum):
    NOT_OPENED = "NOT_OPENED"
    OPENED = "OPENED"


class EnvelopeRecipientCreateManySigningStatus(str, Enum):
    NOT_SIGNED = "NOT_SIGNED"
    SIGNED = "SIGNED"
    REJECTED = "REJECTED"


class EnvelopeRecipientCreateManySendStatus(str, Enum):
    NOT_SENT = "NOT_SENT"
    SENT = "SENT"


class EnvelopeRecipientCreateManyAccessAuthResponse(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class EnvelopeRecipientCreateManyActionAuthResponse(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class EnvelopeRecipientCreateManyAuthOptionsTypedDict(TypedDict):
    access_auth: List[EnvelopeRecipientCreateManyAccessAuthResponse]
    action_auth: List[EnvelopeRecipientCreateManyActionAuthResponse]


class EnvelopeRecipientCreateManyAuthOptions(BaseModel):
    access_auth: Annotated[
        List[EnvelopeRecipientCreateManyAccessAuthResponse],
        pydantic.Field(alias="accessAuth"),
    ]

    action_auth: Annotated[
        List[EnvelopeRecipientCreateManyActionAuthResponse],
        pydantic.Field(alias="actionAuth"),
    ]


class EnvelopeRecipientCreateManyDataResponseTypedDict(TypedDict):
    envelope_id: str
    role: EnvelopeRecipientCreateManyRoleResponse
    read_status: EnvelopeRecipientCreateManyReadStatus
    signing_status: EnvelopeRecipientCreateManySigningStatus
    send_status: EnvelopeRecipientCreateManySendStatus
    id: float
    email: str
    name: str
    token: str
    document_deleted_at: Nullable[str]
    expired: Nullable[str]
    signed_at: Nullable[str]
    auth_options: Nullable[EnvelopeRecipientCreateManyAuthOptionsTypedDict]
    signing_order: Nullable[float]
    rejection_reason: Nullable[str]


class EnvelopeRecipientCreateManyDataResponse(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    role: EnvelopeRecipientCreateManyRoleResponse

    read_status: Annotated[
        EnvelopeRecipientCreateManyReadStatus, pydantic.Field(alias="readStatus")
    ]

    signing_status: Annotated[
        EnvelopeRecipientCreateManySigningStatus, pydantic.Field(alias="signingStatus")
    ]

    send_status: Annotated[
        EnvelopeRecipientCreateManySendStatus, pydantic.Field(alias="sendStatus")
    ]

    id: float

    email: str

    name: str

    token: str

    document_deleted_at: Annotated[
        Nullable[str], pydantic.Field(alias="documentDeletedAt")
    ]

    expired: Nullable[str]

    signed_at: Annotated[Nullable[str], pydantic.Field(alias="signedAt")]

    auth_options: Annotated[
        Nullable[EnvelopeRecipientCreateManyAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    signing_order: Annotated[Nullable[float], pydantic.Field(alias="signingOrder")]

    rejection_reason: Annotated[Nullable[str], pydantic.Field(alias="rejectionReason")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "documentDeletedAt",
            "expired",
            "signedAt",
            "authOptions",
            "signingOrder",
            "rejectionReason",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EnvelopeRecipientCreateManyResponseTypedDict(TypedDict):
    r"""Successful response"""

    data: List[EnvelopeRecipientCreateManyDataResponseTypedDict]


class EnvelopeRecipientCreateManyResponse(BaseModel):
    r"""Successful response"""

    data: List[EnvelopeRecipientCreateManyDataResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from documenso_sdk.utils import FieldMetadata, MultipartFormMetadata
from enum import Enum
import httpx
import io
import pydantic
from pydantic import model_serializer
from typing import Dict, IO, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class DocumentCreateVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class DocumentCreateGlobalAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class DocumentCreateGlobalActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


DocumentCreateFormValuesTypedDict = TypeAliasType(
    "DocumentCreateFormValuesTypedDict", Union[str, bool, float]
)


DocumentCreateFormValues = TypeAliasType(
    "DocumentCreateFormValues", Union[str, bool, float]
)


class DocumentCreateRole(str, Enum):
    CC = "CC"
    SIGNER = "SIGNER"
    VIEWER = "VIEWER"
    APPROVER = "APPROVER"
    ASSISTANT = "ASSISTANT"


class DocumentCreateAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class DocumentCreateActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"
    EXPLICIT_NONE = "EXPLICIT_NONE"


class DocumentCreateTypeDropdown1(str, Enum):
    DROPDOWN = "DROPDOWN"


class DocumentCreateTypeDropdown2(str, Enum):
    DROPDOWN = "dropdown"


class DocumentCreateValueDropdownTypedDict(TypedDict):
    value: str


class DocumentCreateValueDropdown(BaseModel):
    value: str


class DocumentCreateFieldMetaDropdownTypedDict(TypedDict):
    type: DocumentCreateTypeDropdown2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[DocumentCreateValueDropdownTypedDict]]
    default_value: NotRequired[str]


class DocumentCreateFieldMetaDropdown(BaseModel):
    type: DocumentCreateTypeDropdown2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[DocumentCreateValueDropdown]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class DocumentCreateFieldDropdownTypedDict(TypedDict):
    type: DocumentCreateTypeDropdown1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaDropdownTypedDict]


class DocumentCreateFieldDropdown(BaseModel):
    type: DocumentCreateTypeDropdown1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaDropdown], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeCheckbox1(str, Enum):
    CHECKBOX = "CHECKBOX"


class DocumentCreateTypeCheckbox2(str, Enum):
    CHECKBOX = "checkbox"


class DocumentCreateValueCheckboxTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class DocumentCreateValueCheckbox(BaseModel):
    id: float

    checked: bool

    value: str


class DocumentCreateDirectionCheckbox(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class DocumentCreateFieldMetaCheckboxTypedDict(TypedDict):
    type: DocumentCreateTypeCheckbox2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[DocumentCreateValueCheckboxTypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[DocumentCreateDirectionCheckbox]


class DocumentCreateFieldMetaCheckbox(BaseModel):
    type: DocumentCreateTypeCheckbox2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[DocumentCreateValueCheckbox]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[DocumentCreateDirectionCheckbox] = (
        DocumentCreateDirectionCheckbox.VERTICAL
    )


class DocumentCreateFieldCheckboxTypedDict(TypedDict):
    type: DocumentCreateTypeCheckbox1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaCheckboxTypedDict]


class DocumentCreateFieldCheckbox(BaseModel):
    type: DocumentCreateTypeCheckbox1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaCheckbox], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeRadio1(str, Enum):
    RADIO = "RADIO"


class DocumentCreateTypeRadio2(str, Enum):
    RADIO = "radio"


class DocumentCreateValueRadioTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class DocumentCreateValueRadio(BaseModel):
    id: float

    checked: bool

    value: str


class DocumentCreateDirectionRadio(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class DocumentCreateFieldMetaRadioTypedDict(TypedDict):
    type: DocumentCreateTypeRadio2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[DocumentCreateValueRadioTypedDict]]
    direction: NotRequired[DocumentCreateDirectionRadio]


class DocumentCreateFieldMetaRadio(BaseModel):
    type: DocumentCreateTypeRadio2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[DocumentCreateValueRadio]] = None

    direction: Optional[DocumentCreateDirectionRadio] = (
        DocumentCreateDirectionRadio.VERTICAL
    )


class DocumentCreateFieldRadioTypedDict(TypedDict):
    type: DocumentCreateTypeRadio1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaRadioTypedDict]


class DocumentCreateFieldRadio(BaseModel):
    type: DocumentCreateTypeRadio1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaRadio], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeNumber1(str, Enum):
    NUMBER = "NUMBER"


class DocumentCreateTypeNumber2(str, Enum):
    NUMBER = "number"


class DocumentCreateTextAlignNumber(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateVerticalAlignNumber(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class DocumentCreateFieldMetaNumberTypedDict(TypedDict):
    type: DocumentCreateTypeNumber2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[DocumentCreateTextAlignNumber]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[DocumentCreateVerticalAlignNumber]]


class DocumentCreateFieldMetaNumber(BaseModel):
    type: DocumentCreateTypeNumber2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[DocumentCreateTextAlignNumber], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[DocumentCreateVerticalAlignNumber],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class DocumentCreateFieldNumberTypedDict(TypedDict):
    type: DocumentCreateTypeNumber1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaNumberTypedDict]


class DocumentCreateFieldNumber(BaseModel):
    type: DocumentCreateTypeNumber1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaNumber], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeText1(str, Enum):
    TEXT = "TEXT"


class DocumentCreateTypeText2(str, Enum):
    TEXT = "text"


class DocumentCreateTextAlignText(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateVerticalAlignText(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class DocumentCreateFieldMetaTextTypedDict(TypedDict):
    type: DocumentCreateTypeText2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[DocumentCreateTextAlignText]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[DocumentCreateVerticalAlignText]]


class DocumentCreateFieldMetaText(BaseModel):
    type: DocumentCreateTypeText2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[DocumentCreateTextAlignText], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[DocumentCreateVerticalAlignText],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class DocumentCreateFieldTextTypedDict(TypedDict):
    type: DocumentCreateTypeText1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaTextTypedDict]


class DocumentCreateFieldText(BaseModel):
    type: DocumentCreateTypeText1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaText], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeDate1(str, Enum):
    DATE = "DATE"


class DocumentCreateTypeDate2(str, Enum):
    DATE = "date"


class DocumentCreateTextAlignDate(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateFieldMetaDateTypedDict(TypedDict):
    type: DocumentCreateTypeDate2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[DocumentCreateTextAlignDate]


class DocumentCreateFieldMetaDate(BaseModel):
    type: DocumentCreateTypeDate2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[DocumentCreateTextAlignDate], pydantic.Field(alias="textAlign")
    ] = None


class DocumentCreateFieldDateTypedDict(TypedDict):
    type: DocumentCreateTypeDate1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaDateTypedDict]


class DocumentCreateFieldDate(BaseModel):
    type: DocumentCreateTypeDate1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaDate], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeEmail1(str, Enum):
    EMAIL = "EMAIL"


class DocumentCreateTypeEmail2(str, Enum):
    EMAIL = "email"


class DocumentCreateTextAlignEmail(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateFieldMetaEmailTypedDict(TypedDict):
    type: DocumentCreateTypeEmail2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[DocumentCreateTextAlignEmail]


class DocumentCreateFieldMetaEmail(BaseModel):
    type: DocumentCreateTypeEmail2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[DocumentCreateTextAlignEmail], pydantic.Field(alias="textAlign")
    ] = None


class DocumentCreateFieldEmailTypedDict(TypedDict):
    type: DocumentCreateTypeEmail1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaEmailTypedDict]


class DocumentCreateFieldEmail(BaseModel):
    type: DocumentCreateTypeEmail1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaEmail], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeName1(str, Enum):
    NAME = "NAME"


class DocumentCreateTypeName2(str, Enum):
    NAME = "name"


class DocumentCreateTextAlignName(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateFieldMetaNameTypedDict(TypedDict):
    type: DocumentCreateTypeName2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[DocumentCreateTextAlignName]


class DocumentCreateFieldMetaName(BaseModel):
    type: DocumentCreateTypeName2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[DocumentCreateTextAlignName], pydantic.Field(alias="textAlign")
    ] = None


class DocumentCreateFieldNameTypedDict(TypedDict):
    type: DocumentCreateTypeName1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaNameTypedDict]


class DocumentCreateFieldName(BaseModel):
    type: DocumentCreateTypeName1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaName], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeInitials1(str, Enum):
    INITIALS = "INITIALS"


class DocumentCreateTypeInitials2(str, Enum):
    INITIALS = "initials"


class DocumentCreateTextAlignInitials(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class DocumentCreateFieldMetaInitialsTypedDict(TypedDict):
    type: DocumentCreateTypeInitials2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[DocumentCreateTextAlignInitials]


class DocumentCreateFieldMetaInitials(BaseModel):
    type: DocumentCreateTypeInitials2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[DocumentCreateTextAlignInitials], pydantic.Field(alias="textAlign")
    ] = None


class DocumentCreateFieldInitialsTypedDict(TypedDict):
    type: DocumentCreateTypeInitials1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaInitialsTypedDict]


class DocumentCreateFieldInitials(BaseModel):
    type: DocumentCreateTypeInitials1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaInitials], pydantic.Field(alias="fieldMeta")
    ] = None


class DocumentCreateTypeFreeSignature(str, Enum):
    FREE_SIGNATURE = "FREE_SIGNATURE"


class DocumentCreateFieldFreeSignatureTypedDict(TypedDict):
    type: DocumentCreateTypeFreeSignature
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float


class DocumentCreateFieldFreeSignature(BaseModel):
    type: DocumentCreateTypeFreeSignature

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float


class DocumentCreateTypeSignature1(str, Enum):
    SIGNATURE = "SIGNATURE"


class DocumentCreateTypeSignature2(str, Enum):
    SIGNATURE = "signature"


class DocumentCreateFieldMetaSignatureTypedDict(TypedDict):
    type: DocumentCreateTypeSignature2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class DocumentCreateFieldMetaSignature(BaseModel):
    type: DocumentCreateTypeSignature2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


class DocumentCreateFieldSignatureTypedDict(TypedDict):
    type: DocumentCreateTypeSignature1
    page_number: float
    page_x: float
    page_y: float
    width: float
    height: float
    field_meta: NotRequired[DocumentCreateFieldMetaSignatureTypedDict]


class DocumentCreateFieldSignature(BaseModel):
    type: DocumentCreateTypeSignature1

    page_number: Annotated[float, pydantic.Field(alias="pageNumber")]

    page_x: Annotated[float, pydantic.Field(alias="pageX")]

    page_y: Annotated[float, pydantic.Field(alias="pageY")]

    width: float

    height: float

    field_meta: Annotated[
        Optional[DocumentCreateFieldMetaSignature], pydantic.Field(alias="fieldMeta")
    ] = None


DocumentCreateFieldUnionTypedDict = TypeAliasType(
    "DocumentCreateFieldUnionTypedDict",
    Union[
        DocumentCreateFieldFreeSignatureTypedDict,
        DocumentCreateFieldSignatureTypedDict,
        DocumentCreateFieldInitialsTypedDict,
        DocumentCreateFieldNameTypedDict,
        DocumentCreateFieldEmailTypedDict,
        DocumentCreateFieldDateTypedDict,
        DocumentCreateFieldTextTypedDict,
        DocumentCreateFieldNumberTypedDict,
        DocumentCreateFieldRadioTypedDict,
        DocumentCreateFieldCheckboxTypedDict,
        DocumentCreateFieldDropdownTypedDict,
    ],
)


DocumentCreateFieldUnion = TypeAliasType(
    "DocumentCreateFieldUnion",
    Union[
        DocumentCreateFieldFreeSignature,
        DocumentCreateFieldSignature,
        DocumentCreateFieldInitials,
        DocumentCreateFieldName,
        DocumentCreateFieldEmail,
        DocumentCreateFieldDate,
        DocumentCreateFieldText,
        DocumentCreateFieldNumber,
        DocumentCreateFieldRadio,
        DocumentCreateFieldCheckbox,
        DocumentCreateFieldDropdown,
    ],
)


class DocumentCreateRecipientTypedDict(TypedDict):
    email: str
    name: str
    role: DocumentCreateRole
    signing_order: NotRequired[float]
    access_auth: NotRequired[List[DocumentCreateAccessAuth]]
    action_auth: NotRequired[List[DocumentCreateActionAuth]]
    fields: NotRequired[List[DocumentCreateFieldUnionTypedDict]]


class DocumentCreateRecipient(BaseModel):
    email: str

    name: str

    role: DocumentCreateRole

    signing_order: Annotated[Optional[float], pydantic.Field(alias="signingOrder")] = (
        None
    )

    access_auth: Annotated[
        Optional[List[DocumentCreateAccessAuth]], pydantic.Field(alias="accessAuth")
    ] = None

    action_auth: Annotated[
        Optional[List[DocumentCreateActionAuth]], pydantic.Field(alias="actionAuth")
    ] = None

    fields: Optional[List[DocumentCreateFieldUnion]] = None


class DocumentCreateTypeLink(str, Enum):
    LINK = "link"


class DocumentCreateAttachmentTypedDict(TypedDict):
    label: str
    data: str
    type: NotRequired[DocumentCreateTypeLink]


class DocumentCreateAttachment(BaseModel):
    label: str

    data: str

    type: Optional[DocumentCreateTypeLink] = DocumentCreateTypeLink.LINK


class DocumentCreateDateFormat(str, Enum):
    YYYY_M_MDD_HH_MM_A = "yyyy-MM-dd hh:mm a"
    YYYY_M_MDD = "yyyy-MM-dd"
    DD_MM_SLASH_YYYY = "dd/MM/yyyy"
    MM_DD_SLASH_YYYY = "MM/dd/yyyy"
    YY_M_MDD = "yy-MM-dd"
    MMMM_DD_COMMA_YYYY = "MMMM dd, yyyy"
    EEEE_MMMM_DD_COMMA_YYYY = "EEEE, MMMM dd, yyyy"
    DD_MM_SLASH_YYYY_HH_MM_A = "dd/MM/yyyy hh:mm a"
    DD_MM_SLASH_YYYY_H_HMM = "dd/MM/yyyy HH:mm"
    MM_DD_SLASH_YYYY_HH_MM_A = "MM/dd/yyyy hh:mm a"
    MM_DD_SLASH_YYYY_H_HMM = "MM/dd/yyyy HH:mm"
    DD_DOT_MM_DOT_YYYY = "dd.MM.yyyy"
    DD_DOT_MM_DOT_YYYY_H_HMM = "dd.MM.yyyy HH:mm"
    YYYY_M_MDD_H_HMM = "yyyy-MM-dd HH:mm"
    YY_M_MDD_HH_MM_A = "yy-MM-dd hh:mm a"
    YY_M_MDD_H_HMM = "yy-MM-dd HH:mm"
    YYYY_M_MDD_H_HMMSS = "yyyy-MM-dd HH:mm:ss"
    MMMM_DD_COMMA_YYYY_HH_MM_A = "MMMM dd, yyyy hh:mm a"
    MMMM_DD_COMMA_YYYY_H_HMM = "MMMM dd, yyyy HH:mm"
    EEEE_MMMM_DD_COMMA_YYYY_HH_MM_A = "EEEE, MMMM dd, yyyy hh:mm a"
    EEEE_MMMM_DD_COMMA_YYYY_H_HMM = "EEEE, MMMM dd, yyyy HH:mm"
    ISO8601_FULL = "yyyy-MM-dd'T'HH:mm:ss.SSSXXX"


class DocumentCreateDistributionMethod(str, Enum):
    EMAIL = "EMAIL"
    NONE = "NONE"


class DocumentCreateSigningOrder(str, Enum):
    PARALLEL = "PARALLEL"
    SEQUENTIAL = "SEQUENTIAL"


class DocumentCreateLanguage(str, Enum):
    DE = "de"
    EN = "en"
    FR = "fr"
    ES = "es"
    IT = "it"
    PL = "pl"


class DocumentCreateEmailSettingsTypedDict(TypedDict):
    recipient_signing_request: NotRequired[bool]
    recipient_removed: NotRequired[bool]
    recipient_signed: NotRequired[bool]
    document_pending: NotRequired[bool]
    document_completed: NotRequired[bool]
    document_deleted: NotRequired[bool]
    owner_document_completed: NotRequired[bool]


class DocumentCreateEmailSettings(BaseModel):
    recipient_signing_request: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigningRequest")
    ] = True

    recipient_removed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientRemoved")
    ] = True

    recipient_signed: Annotated[
        Optional[bool], pydantic.Field(alias="recipientSigned")
    ] = True

    document_pending: Annotated[
        Optional[bool], pydantic.Field(alias="documentPending")
    ] = True

    document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="documentCompleted")
    ] = True

    document_deleted: Annotated[
        Optional[bool], pydantic.Field(alias="documentDeleted")
    ] = True

    owner_document_completed: Annotated[
        Optional[bool], pydantic.Field(alias="ownerDocumentCompleted")
    ] = True


class DocumentCreateMetaTypedDict(TypedDict):
    subject: NotRequired[str]
    message: NotRequired[str]
    timezone: NotRequired[str]
    date_format: NotRequired[DocumentCreateDateFormat]
    distribution_method: NotRequired[DocumentCreateDistributionMethod]
    signing_order: NotRequired[DocumentCreateSigningOrder]
    allow_dictate_next_signer: NotRequired[bool]
    redirect_url: NotRequired[str]
    language: NotRequired[DocumentCreateLanguage]
    typed_signature_enabled: NotRequired[bool]
    upload_signature_enabled: NotRequired[bool]
    draw_signature_enabled: NotRequired[bool]
    email_id: NotRequired[Nullable[str]]
    email_reply_to: NotRequired[Nullable[str]]
    email_settings: NotRequired[Nullable[DocumentCreateEmailSettingsTypedDict]]


class DocumentCreateMeta(BaseModel):
    subject: Optional[str] = None

    message: Optional[str] = None

    timezone: Optional[str] = None

    date_format: Annotated[
        Optional[DocumentCreateDateFormat], pydantic.Field(alias="dateFormat")
    ] = None

    distribution_method: Annotated[
        Optional[DocumentCreateDistributionMethod],
        pydantic.Field(alias="distributionMethod"),
    ] = None

    signing_order: Annotated[
        Optional[DocumentCreateSigningOrder], pydantic.Field(alias="signingOrder")
    ] = None

    allow_dictate_next_signer: Annotated[
        Optional[bool], pydantic.Field(alias="allowDictateNextSigner")
    ] = None

    redirect_url: Annotated[Optional[str], pydantic.Field(alias="redirectUrl")] = None

    language: Optional[DocumentCreateLanguage] = None

    typed_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="typedSignatureEnabled")
    ] = None

    upload_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="uploadSignatureEnabled")
    ] = None

    draw_signature_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="drawSignatureEnabled")
    ] = None

    email_id: Annotated[OptionalNullable[str], pydantic.Field(alias="emailId")] = UNSET

    email_reply_to: Annotated[
        OptionalNullable[str], pydantic.Field(alias="emailReplyTo")
    ] = UNSET

    email_settings: Annotated[
        OptionalNullable[DocumentCreateEmailSettings],
        pydantic.Field(alias="emailSettings"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "subject",
            "message",
            "timezone",
            "dateFormat",
            "distributionMethod",
            "signingOrder",
            "allowDictateNextSigner",
            "redirectUrl",
            "language",
            "typedSignatureEnabled",
            "uploadSignatureEnabled",
            "drawSignatureEnabled",
            "emailId",
            "emailReplyTo",
            "emailSettings",
        ]
        nullable_fields = ["emailId", "emailReplyTo", "emailSettings"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class DocumentCreatePayloadTypedDict(TypedDict):
    title: str
    external_id: NotRequired[str]
    visibility: NotRequired[DocumentCreateVisibility]
    global_access_auth: NotRequired[List[DocumentCreateGlobalAccessAuth]]
    global_action_auth: NotRequired[List[DocumentCreateGlobalActionAuth]]
    form_values: NotRequired[Dict[str, DocumentCreateFormValuesTypedDict]]
    folder_id: NotRequired[str]
    recipients: NotRequired[List[DocumentCreateRecipientTypedDict]]
    attachments: NotRequired[List[DocumentCreateAttachmentTypedDict]]
    meta: NotRequired[DocumentCreateMetaTypedDict]


class DocumentCreatePayload(BaseModel):
    title: str

    external_id: Annotated[Optional[str], pydantic.Field(alias="externalId")] = None

    visibility: Optional[DocumentCreateVisibility] = None

    global_access_auth: Annotated[
        Optional[List[DocumentCreateGlobalAccessAuth]],
        pydantic.Field(alias="globalAccessAuth"),
    ] = None

    global_action_auth: Annotated[
        Optional[List[DocumentCreateGlobalActionAuth]],
        pydantic.Field(alias="globalActionAuth"),
    ] = None

    form_values: Annotated[
        Optional[Dict[str, DocumentCreateFormValues]],
        pydantic.Field(alias="formValues"),
    ] = None

    folder_id: Annotated[Optional[str], pydantic.Field(alias="folderId")] = None

    recipients: Optional[List[DocumentCreateRecipient]] = None

    attachments: Optional[List[DocumentCreateAttachment]] = None

    meta: Optional[DocumentCreateMeta] = None


class DocumentCreateFileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class DocumentCreateFile(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


class DocumentCreateRequestTypedDict(TypedDict):
    payload: DocumentCreatePayloadTypedDict
    file: DocumentCreateFileTypedDict


class DocumentCreateRequest(BaseModel):
    payload: Annotated[
        DocumentCreatePayload, FieldMetadata(multipart=MultipartFormMetadata(json=True))
    ]

    file: Annotated[
        DocumentCreateFile, FieldMetadata(multipart=MultipartFormMetadata(file=True))
    ]


class DocumentCreateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class DocumentCreateInternalServerErrorIssue(BaseModel):
    message: str


class DocumentCreateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentCreateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentCreateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: DocumentCreateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentCreateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentCreateForbiddenIssueTypedDict(TypedDict):
    message: str


class DocumentCreateForbiddenIssue(BaseModel):
    message: str


class DocumentCreateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentCreateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentCreateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: DocumentCreateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentCreateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentCreateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class DocumentCreateUnauthorizedIssue(BaseModel):
    message: str


class DocumentCreateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentCreateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentCreateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: DocumentCreateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentCreateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentCreateBadRequestIssueTypedDict(TypedDict):
    message: str


class DocumentCreateBadRequestIssue(BaseModel):
    message: str


class DocumentCreateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentCreateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentCreateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: DocumentCreateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentCreateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentCreateResponseTypedDict(TypedDict):
    r"""Successful response"""

    envelope_id: str
    id: float


class DocumentCreateResponse(BaseModel):
    r"""Successful response"""

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    id: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EnvelopeItemUpdateManyDataRequestTypedDict(TypedDict):
    envelope_item_id: str
    order: NotRequired[int]
    title: NotRequired[str]


class EnvelopeItemUpdateManyDataRequest(BaseModel):
    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    order: Optional[int] = None

    title: Optional[str] = None


class EnvelopeItemUpdateManyRequestTypedDict(TypedDict):
    envelope_id: str
    data: List[EnvelopeItemUpdateManyDataRequestTypedDict]


class EnvelopeItemUpdateManyRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    data: List[EnvelopeItemUpdateManyDataRequest]


class EnvelopeItemUpdateManyInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemUpdateManyInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeItemUpdateManyInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemUpdateManyInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemUpdateManyInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeItemUpdateManyInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemUpdateManyInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemUpdateManyForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemUpdateManyForbiddenIssue(BaseModel):
    message: str


class EnvelopeItemUpdateManyForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemUpdateManyForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemUpdateManyForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeItemUpdateManyForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemUpdateManyForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemUpdateManyUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemUpdateManyUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeItemUpdateManyUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemUpdateManyUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemUpdateManyUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeItemUpdateManyUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemUpdateManyUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemUpdateManyBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemUpdateManyBadRequestIssue(BaseModel):
    message: str


class EnvelopeItemUpdateManyBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemUpdateManyBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemUpdateManyBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeItemUpdateManyBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemUpdateManyBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemUpdateManyDataResponseTypedDict(TypedDict):
    id: str
    order: float
    title: str
    envelope_id: str


class EnvelopeItemUpdateManyDataResponse(BaseModel):
    id: str

    order: float

    title: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class EnvelopeItemUpdateManyResponseTypedDict(TypedDict):
    r"""Successful response"""

    data: List[EnvelopeItemUpdateManyDataResponseTypedDict]


class EnvelopeItemUpdateManyResponse(BaseModel):
    r"""Successful response"""

    data: List[EnvelopeItemUpdateManyDataResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from documenso_sdk import models, utils
from documenso_sdk._hooks import HookContext
from documenso_sdk.types import OptionalNullable, UNSET
from documenso_sdk.utils import get_security_from_env
from documenso_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class DocumentsRecipients(BaseSDK):
    def get(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientGetDocumentRecipientResponse:
        r"""Get document recipient

        Returns a single recipient. If you want to retrieve all the recipients for a document, use the \"Get Document\" endpoint.

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientGetDocumentRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request(
            method="GET",
            path="/document/recipient/{recipientId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-getDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientGetDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientNotFoundErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientNotFoundError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientGetDocumentRecipientResponse:
        r"""Get document recipient

        Returns a single recipient. If you want to retrieve all the recipients for a document, use the \"Get Document\" endpoint.

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientGetDocumentRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/document/recipient/{recipientId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-getDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientGetDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientNotFoundErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientNotFoundError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientGetDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientGetDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        document_id: float,
        recipient: Union[
            models.RecipientCreateDocumentRecipientRecipient,
            models.RecipientCreateDocumentRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateDocumentRecipientResponse:
        r"""Create document recipient

        Create a single recipient for a document.

        :param document_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateDocumentRecipientRequest(
            document_id=document_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientCreateDocumentRecipientRecipient
            ),
        )

        req = self._build_request(
            method="POST",
            path="/document/recipient/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        document_id: float,
        recipient: Union[
            models.RecipientCreateDocumentRecipientRecipient,
            models.RecipientCreateDocumentRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateDocumentRecipientResponse:
        r"""Create document recipient

        Create a single recipient for a document.

        :param document_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateDocumentRecipientRequest(
            document_id=document_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientCreateDocumentRecipientRecipient
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/document/recipient/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create_many(
        self,
        *,
        document_id: float,
        recipients: Union[
            List[models.RecipientCreateDocumentRecipientsRecipientRequest],
            List[models.RecipientCreateDocumentRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateDocumentRecipientsResponse:
        r"""Create document recipients

        Create multiple recipients for a document.

        :param document_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateDocumentRecipientsRequest(
            document_id=document_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientCreateDocumentRecipientsRecipientRequest],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/document/recipient/create-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateDocumentRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createDocumentRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientCreateDocumentRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_many_async(
        self,
        *,
        document_id: float,
        recipients: Union[
            List[models.RecipientCreateDocumentRecipientsRecipientRequest],
            List[models.RecipientCreateDocumentRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientCreateDocumentRecipientsResponse:
        r"""Create document recipients

        Create multiple recipients for a document.

        :param document_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientCreateDocumentRecipientsRequest(
            document_id=document_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientCreateDocumentRecipientsRecipientRequest],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/document/recipient/create-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientCreateDocumentRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-createDocumentRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientCreateDocumentRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientCreateDocumentRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientCreateDocumentRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        document_id: float,
        recipient: Union[
            models.RecipientUpdateDocumentRecipientRecipient,
            models.RecipientUpdateDocumentRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateDocumentRecipientResponse:
        r"""Update document recipient

        Update a single recipient for a document.

        :param document_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateDocumentRecipientRequest(
            document_id=document_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientUpdateDocumentRecipientRecipient
            ),
        )

        req = self._build_request(
            method="POST",
            path="/document/recipient/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        document_id: float,
        recipient: Union[
            models.RecipientUpdateDocumentRecipientRecipient,
            models.RecipientUpdateDocumentRecipientRecipientTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateDocumentRecipientResponse:
        r"""Update document recipient

        Update a single recipient for a document.

        :param document_id:
        :param recipient:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateDocumentRecipientRequest(
            document_id=document_id,
            recipient=utils.get_pydantic_model(
                recipient, models.RecipientUpdateDocumentRecipientRecipient
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/document/recipient/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update_many(
        self,
        *,
        document_id: float,
        recipients: Union[
            List[models.RecipientUpdateDocumentRecipientsRecipientRequest],
            List[models.RecipientUpdateDocumentRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateDocumentRecipientsResponse:
        r"""Update document recipients

        Update multiple recipients for a document.

        :param document_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateDocumentRecipientsRequest(
            document_id=document_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientUpdateDocumentRecipientsRecipientRequest],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/document/recipient/update-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateDocumentRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateDocumentRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientUpdateDocumentRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_many_async(
        self,
        *,
        document_id: float,
        recipients: Union[
            List[models.RecipientUpdateDocumentRecipientsRecipientRequest],
            List[models.RecipientUpdateDocumentRecipientsRecipientRequestTypedDict],
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientUpdateDocumentRecipientsResponse:
        r"""Update document recipients

        Update multiple recipients for a document.

        :param document_id:
        :param recipients:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientUpdateDocumentRecipientsRequest(
            document_id=document_id,
            recipients=utils.get_pydantic_model(
                recipients,
                List[models.RecipientUpdateDocumentRecipientsRecipientRequest],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/document/recipient/update-many",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientUpdateDocumentRecipientsRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-updateDocumentRecipients",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsBadRequestErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsUnauthorizedErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsForbiddenErrorData, http_res
            )
            raise models.RecipientUpdateDocumentRecipientsForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientUpdateDocumentRecipientsInternalServerErrorData,
                http_res,
            )
            raise models.RecipientUpdateDocumentRecipientsInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientDeleteDocumentRecipientResponse:
        r"""Delete document recipient

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientDeleteDocumentRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request(
            method="POST",
            path="/document/recipient/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientDeleteDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-deleteDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        recipient_id: float,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.RecipientDeleteDocumentRecipientResponse:
        r"""Delete document recipient

        :param recipient_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.RecipientDeleteDocumentRecipientRequest(
            recipient_id=recipient_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/document/recipient/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.RecipientDeleteDocumentRecipientRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="recipient-deleteDocumentRecipient",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientBadRequestErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientUnauthorizedErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientForbiddenErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.RecipientDeleteDocumentRecipientInternalServerErrorData, http_res
            )
            raise models.RecipientDeleteDocumentRecipientInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from documenso_sdk.utils import FieldMetadata, PathParamMetadata
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class EnvelopeFieldGetRequestTypedDict(TypedDict):
    field_id: float


class EnvelopeFieldGetRequest(BaseModel):
    field_id: Annotated[
        float,
        pydantic.Field(alias="fieldId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]


class EnvelopeFieldGetInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeFieldGetInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeFieldGetInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeFieldGetInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeFieldGetInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeFieldGetInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeFieldGetInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeFieldGetNotFoundIssueTypedDict(TypedDict):
    message: str


class EnvelopeFieldGetNotFoundIssue(BaseModel):
    message: str


class EnvelopeFieldGetNotFoundErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeFieldGetNotFoundIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeFieldGetNotFoundError(DocumensoError):
    r"""Not found"""

    data: EnvelopeFieldGetNotFoundErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeFieldGetNotFoundErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeFieldGetForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeFieldGetForbiddenIssue(BaseModel):
    message: str


class EnvelopeFieldGetForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeFieldGetForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeFieldGetForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeFieldGetForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeFieldGetForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeFieldGetUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeFieldGetUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeFieldGetUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeFieldGetUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeFieldGetUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeFieldGetUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeFieldGetUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeFieldGetBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeFieldGetBadRequestIssue(BaseModel):
    message: str


class EnvelopeFieldGetBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeFieldGetBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeFieldGetBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeFieldGetBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeFieldGetBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeFieldGetType(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class EnvelopeFieldGetTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class EnvelopeFieldGetValue3TypedDict(TypedDict):
    value: str


class EnvelopeFieldGetValue3(BaseModel):
    value: str


class EnvelopeFieldGetFieldMetaDropdownTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[EnvelopeFieldGetValue3TypedDict]]
    default_value: NotRequired[str]


class EnvelopeFieldGetFieldMetaDropdown(BaseModel):
    type: EnvelopeFieldGetTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[EnvelopeFieldGetValue3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class EnvelopeFieldGetTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class EnvelopeFieldGetValue2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class EnvelopeFieldGetValue2(BaseModel):
    id: float

    checked: bool

    value: str


class EnvelopeFieldGetDirection2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class EnvelopeFieldGetFieldMetaCheckboxTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[EnvelopeFieldGetValue2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[EnvelopeFieldGetDirection2]


class EnvelopeFieldGetFieldMetaCheckbox(BaseModel):
    type: EnvelopeFieldGetTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[EnvelopeFieldGetValue2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[EnvelopeFieldGetDirection2] = (
        EnvelopeFieldGetDirection2.VERTICAL
    )


class EnvelopeFieldGetTypeRadio(str, Enum):
    RADIO = "radio"


class EnvelopeFieldGetValue1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class EnvelopeFieldGetValue1(BaseModel):
    id: float

    checked: bool

    value: str


class EnvelopeFieldGetDirection1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class EnvelopeFieldGetFieldMetaRadioTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[EnvelopeFieldGetValue1TypedDict]]
    direction: NotRequired[EnvelopeFieldGetDirection1]


class EnvelopeFieldGetFieldMetaRadio(BaseModel):
    type: EnvelopeFieldGetTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[EnvelopeFieldGetValue1]] = None

    direction: Optional[EnvelopeFieldGetDirection1] = (
        EnvelopeFieldGetDirection1.VERTICAL
    )


class EnvelopeFieldGetTypeNumber(str, Enum):
    NUMBER = "number"


class EnvelopeFieldGetTextAlign6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetVerticalAlign2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class EnvelopeFieldGetFieldMetaNumberTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[EnvelopeFieldGetTextAlign6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[EnvelopeFieldGetVerticalAlign2]]


class EnvelopeFieldGetFieldMetaNumber(BaseModel):
    type: EnvelopeFieldGetTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign6], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[EnvelopeFieldGetVerticalAlign2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EnvelopeFieldGetTypeText(str, Enum):
    TEXT = "text"


class EnvelopeFieldGetTextAlign5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetVerticalAlign1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class EnvelopeFieldGetFieldMetaTextTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[EnvelopeFieldGetTextAlign5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[EnvelopeFieldGetVerticalAlign1]]


class EnvelopeFieldGetFieldMetaText(BaseModel):
    type: EnvelopeFieldGetTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign5], pydantic.Field(alias="textAlign")
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[EnvelopeFieldGetVerticalAlign1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EnvelopeFieldGetTypeDate(str, Enum):
    DATE = "date"


class EnvelopeFieldGetTextAlign4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetFieldMetaDateTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[EnvelopeFieldGetTextAlign4]


class EnvelopeFieldGetFieldMetaDate(BaseModel):
    type: EnvelopeFieldGetTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign4], pydantic.Field(alias="textAlign")
    ] = None


class EnvelopeFieldGetTypeEmail(str, Enum):
    EMAIL = "email"


class EnvelopeFieldGetTextAlign3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetFieldMetaEmailTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[EnvelopeFieldGetTextAlign3]


class EnvelopeFieldGetFieldMetaEmail(BaseModel):
    type: EnvelopeFieldGetTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign3], pydantic.Field(alias="textAlign")
    ] = None


class EnvelopeFieldGetTypeName(str, Enum):
    NAME = "name"


class EnvelopeFieldGetTextAlign2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetFieldMetaNameTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[EnvelopeFieldGetTextAlign2]


class EnvelopeFieldGetFieldMetaName(BaseModel):
    type: EnvelopeFieldGetTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign2], pydantic.Field(alias="textAlign")
    ] = None


class EnvelopeFieldGetTypeInitials(str, Enum):
    INITIALS = "initials"


class EnvelopeFieldGetTextAlign1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class EnvelopeFieldGetFieldMetaInitialsTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[EnvelopeFieldGetTextAlign1]


class EnvelopeFieldGetFieldMetaInitials(BaseModel):
    type: EnvelopeFieldGetTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[EnvelopeFieldGetTextAlign1], pydantic.Field(alias="textAlign")
    ] = None


class EnvelopeFieldGetTypeSignature(str, Enum):
    SIGNATURE = "signature"


class EnvelopeFieldGetFieldMetaSignatureTypedDict(TypedDict):
    type: EnvelopeFieldGetTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class EnvelopeFieldGetFieldMetaSignature(BaseModel):
    type: EnvelopeFieldGetTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


EnvelopeFieldGetFieldMetaUnionTypedDict = TypeAliasType(
    "EnvelopeFieldGetFieldMetaUnionTypedDict",
    Union[
        EnvelopeFieldGetFieldMetaSignatureTypedDict,
        EnvelopeFieldGetFieldMetaInitialsTypedDict,
        EnvelopeFieldGetFieldMetaNameTypedDict,
        EnvelopeFieldGetFieldMetaEmailTypedDict,
        EnvelopeFieldGetFieldMetaDateTypedDict,
        EnvelopeFieldGetFieldMetaRadioTypedDict,
        EnvelopeFieldGetFieldMetaDropdownTypedDict,
        EnvelopeFieldGetFieldMetaCheckboxTypedDict,
        EnvelopeFieldGetFieldMetaTextTypedDict,
        EnvelopeFieldGetFieldMetaNumberTypedDict,
    ],
)


EnvelopeFieldGetFieldMetaUnion = TypeAliasType(
    "EnvelopeFieldGetFieldMetaUnion",
    Union[
        EnvelopeFieldGetFieldMetaSignature,
        EnvelopeFieldGetFieldMetaInitials,
        EnvelopeFieldGetFieldMetaName,
        EnvelopeFieldGetFieldMetaEmail,
        EnvelopeFieldGetFieldMetaDate,
        EnvelopeFieldGetFieldMetaRadio,
        EnvelopeFieldGetFieldMetaDropdown,
        EnvelopeFieldGetFieldMetaCheckbox,
        EnvelopeFieldGetFieldMetaText,
        EnvelopeFieldGetFieldMetaNumber,
    ],
)


class EnvelopeFieldGetResponseTypedDict(TypedDict):
    r"""Successful response"""

    envelope_id: str
    envelope_item_id: str
    type: EnvelopeFieldGetType
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[EnvelopeFieldGetFieldMetaUnionTypedDict]


class EnvelopeFieldGetResponse(BaseModel):
    r"""Successful response"""

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: EnvelopeFieldGetType

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[EnvelopeFieldGetFieldMetaUnion], pydantic.Field(alias="fieldMeta")
    ]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["fieldMeta"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
from documenso_sdk.utils import FieldMetadata, MultipartFormMetadata
import httpx
import io
import pydantic
from typing import IO, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class EnvelopeItemCreateManyPayloadTypedDict(TypedDict):
    envelope_id: str


class EnvelopeItemCreateManyPayload(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]


class EnvelopeItemCreateManyFileTypedDict(TypedDict):
    file_name: str
    content: Union[bytes, IO[bytes], io.BufferedReader]
    content_type: NotRequired[str]


class EnvelopeItemCreateManyFile(BaseModel):
    file_name: Annotated[
        str, pydantic.Field(alias="fileName"), FieldMetadata(multipart=True)
    ]

    content: Annotated[
        Union[bytes, IO[bytes], io.BufferedReader],
        pydantic.Field(alias=""),
        FieldMetadata(multipart=MultipartFormMetadata(content=True)),
    ]

    content_type: Annotated[
        Optional[str],
        pydantic.Field(alias="Content-Type"),
        FieldMetadata(multipart=True),
    ] = None


class EnvelopeItemCreateManyRequestTypedDict(TypedDict):
    payload: EnvelopeItemCreateManyPayloadTypedDict
    files: NotRequired[List[EnvelopeItemCreateManyFileTypedDict]]


class EnvelopeItemCreateManyRequest(BaseModel):
    payload: Annotated[
        EnvelopeItemCreateManyPayload,
        FieldMetadata(multipart=MultipartFormMetadata(json=True)),
    ]

    files: Annotated[
        Optional[List[EnvelopeItemCreateManyFile]],
        FieldMetadata(multipart=MultipartFormMetadata(file=True)),
    ] = None


class EnvelopeItemCreateManyInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemCreateManyInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeItemCreateManyInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemCreateManyInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemCreateManyInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeItemCreateManyInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemCreateManyInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemCreateManyForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemCreateManyForbiddenIssue(BaseModel):
    message: str


class EnvelopeItemCreateManyForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemCreateManyForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemCreateManyForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeItemCreateManyForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemCreateManyForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemCreateManyUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemCreateManyUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeItemCreateManyUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemCreateManyUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemCreateManyUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeItemCreateManyUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemCreateManyUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemCreateManyBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeItemCreateManyBadRequestIssue(BaseModel):
    message: str


class EnvelopeItemCreateManyBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeItemCreateManyBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeItemCreateManyBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeItemCreateManyBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeItemCreateManyBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeItemCreateManyDataTypedDict(TypedDict):
    id: str
    title: str
    envelope_id: str
    order: float


class EnvelopeItemCreateManyData(BaseModel):
    id: str

    title: str

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    order: float


class EnvelopeItemCreateManyResponseTypedDict(TypedDict):
    r"""Successful response"""

    data: List[EnvelopeItemCreateManyDataTypedDict]


class EnvelopeItemCreateManyResponse(BaseModel):
    r"""Successful response"""

    data: List[EnvelopeItemCreateManyData]
